/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef QUB_BASELINESEG_H
#define QUB_BASELINESEG_H

typedef struct QUB_BaselineSeg_Node {
	int point;
	int value;
	std::map<int, struct QUB_BaselineSeg_Node*>::iterator iter;
} QUB_BaselineSeg_Node;

typedef struct {
	int first;
	int last;
	std::map<int, QUB_BaselineSeg_Node*> *nodes;
} QUB_BaselineSeg;

extern "C" MIL_API QUB_BaselineSeg *       __stdcall QUB_BaselineSeg_Create( int first, int last );
extern "C" MIL_API void                    __stdcall QUB_BaselineSeg_Free( QUB_BaselineSeg *seg );
extern "C" MIL_API int                     __stdcall QUB_BaselineSeg_GetNodeCount( QUB_BaselineSeg *seg );
extern "C" MIL_API void                    __stdcall QUB_BaselineSeg_SetFirst( QUB_BaselineSeg *seg, int first );
extern "C" MIL_API void                    __stdcall QUB_BaselineSeg_SetLast( QUB_BaselineSeg *seg, int last );
extern "C" MIL_API QUB_BaselineSeg *       __stdcall QUB_BaselineSeg_ExciseFrom( QUB_BaselineSeg *seg, int first );
extern "C" MIL_API QUB_BaselineSeg_Node *  __stdcall QUB_BaselineSeg_GetFirstNode( QUB_BaselineSeg *seg );
extern "C" MIL_API QUB_BaselineSeg_Node *  __stdcall QUB_BaselineSeg_GetLastNode( QUB_BaselineSeg *seg );
extern "C" MIL_API QUB_BaselineSeg_Node *  __stdcall QUB_BaselineSeg_GetNodeBefore( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node *node );
extern "C" MIL_API QUB_BaselineSeg_Node *  __stdcall QUB_BaselineSeg_GetNodeAfter( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node *node );

extern "C" MIL_API int                     __stdcall QUB_BaselineSeg_InterpolateValueAt( QUB_BaselineSeg *seg, int point );
// next three: return non-zero (negative) for error e.g. out-of-range / past-the-end
extern "C" MIL_API int                     __stdcall QUB_BaselineSeg_GetRangeAt( QUB_BaselineSeg *seg, int point, QUB_BaselineSeg_Node **from, QUB_BaselineSeg_Node **to );
extern "C" MIL_API int                     __stdcall QUB_BaselineSeg_GetRangeBefore( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node **from, QUB_BaselineSeg_Node **to );
extern "C" MIL_API int                     __stdcall QUB_BaselineSeg_GetRangeAfter( QUB_BaselineSeg *seg, QUB_BaselineSeg_Node **from, QUB_BaselineSeg_Node **to );
// existing nodes at the point / in the range given will be cleared first
extern "C" MIL_API QUB_BaselineSeg_Node *  __stdcall QUB_BaselineSeg_AddNode( QUB_BaselineSeg *seg, int point, int value );
extern "C" MIL_API QUB_BaselineSeg_Node *  __stdcall QUB_BaselineSeg_AddNodes( QUB_BaselineSeg *seg, int count, int *points, int *values );
extern "C" MIL_API void                    __stdcall QUB_BaselineSeg_ClearNodes( QUB_BaselineSeg *seg, int first, int last, int repairFirstLast );

extern "C" MIL_API void                    __stdcall QUB_BaselineSeg_Offset( QUB_BaselineSeg *seg, int points );
extern "C" MIL_API void                    __stdcall QUB_BaselineSeg_Insert( QUB_BaselineSeg *seg, int before, int points );


#endif
